package org.jboss.cache.marshall;

import org.jboss.cache.Fqn;
import org.jboss.cache.StringFqn;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * An evolution of {@link CacheMarshaller210}, to allow for {@link org.jboss.cache.StringFqn}s.
 *
 * @author Manik Surtani
 * @since 2.2.0
 */
public class CacheMarshaller220 extends CacheMarshaller210
{
   public CacheMarshaller220()
   {
      initLogger();
      // disabled, since this is always disabled in JBC 2.0.0.
      // Java ObjectOutputStreams will have it's own built-in ref counting.  No need to repeat this.
      useRefs = false;
   }

   @Override
   protected void marshallFqn(Fqn fqn, ObjectOutputStream out, Map<Object, Integer> refMap) throws Exception
   {
      boolean isRoot = fqn.isRoot();
      out.writeBoolean(isRoot);
      if (!isRoot)
      {
         if (fqn.getClass().equals(StringFqn.class))
         {
            out.writeShort(-1);
            marshallString(((StringFqn) fqn).getStringRepresentation(), out);
         }
         else
         {
            out.writeShort(fqn.size());
            for (Object o : fqn.peekElements())
            {
               marshallObject(o, out, refMap);
            }
         }
      }
   }

   @Override
   protected Fqn unmarshallFqn(ObjectInputStream in, UnmarshalledReferences refMap) throws Exception
   {

      boolean isRoot = in.readBoolean();
      Fqn fqn;
      if (!isRoot)
      {
         int numElements = in.readShort();
         if (numElements == -1)
         {
            // this is a "new" StringFqn.
            fqn = Fqn.fromString(unmarshallString(in));
         }
         else
         {
            // an "old-style" mixed-type Fqn
            List<Object> elements = new ArrayList<Object>(numElements);
            for (int i = 0; i < numElements; i++)
            {
               elements.add(unmarshallObject(in, refMap));
            }
            fqn = Fqn.fromList(elements);
         }
      }
      else
      {
         fqn = Fqn.ROOT;
      }
      return fqn;
   }
}