package org.jboss.cache.util.reflect;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

/**
 * A cached Method object, so that calls to getParameterTypes, getAnnotations, etc are cached.
 *
 * @author Manik Surtani (<a href="mailto:manik@jboss.org">manik@jboss.org</a>)
 * @since 2.1.0
 */
public class CachedMethod
{
   Method method;
   Class[] parameterTypes;
   Annotation[][] parameterAnnotations;

   public CachedMethod(Method method)
   {
      this.method = method;
      this.parameterTypes = method.getParameterTypes();
      this.parameterAnnotations = method.getParameterAnnotations();
   }

   public Method getMethod()
   {
      return method;
   }

   public Class[] getParameterTypes()
   {
      return parameterTypes;
   }

   public Annotation[][] getParameterAnnotations()
   {
      return parameterAnnotations;
   }
}
