package org.jboss.cache.notifications;

import java.util.ArrayList;
import java.util.List;

import org.jboss.cache.notifications.annotation.CacheListener;
import org.jboss.cache.notifications.annotation.NodeCreated;
import org.jboss.cache.notifications.annotation.NodeModified;
import org.jboss.cache.notifications.annotation.NodeMoved;
import org.jboss.cache.notifications.annotation.NodeRemoved;
import org.jboss.cache.notifications.annotation.NodeVisited;
import org.jboss.cache.notifications.annotation.TransactionCompleted;
import org.jboss.cache.notifications.annotation.TransactionRegistered;
import org.jboss.cache.notifications.event.Event;
import org.jboss.cache.notifications.event.EventImpl;

@CacheListener
public class EventLog
{
   public final List<Event> events = new ArrayList<Event>();

   @NodeCreated
   @NodeRemoved
   @NodeModified
   @NodeVisited
   @NodeMoved
   @TransactionCompleted
   @TransactionRegistered
   public void callback(Event e)
   {
      events.add(e);
   }

   public String toString()
   {
      return "EventLog{" +
              "events=" + events +
              '}';
   }

   /**
    * Done when we don't have a Transaction reference to compare with, e.g., when using implicit transactions in
    * opt locking.
    */
   public void scrubImplicitTransactions()
   {
      for (Event e : events) ((EventImpl) e).setTransaction(null);
   }
}